#include "Game.h"

Game::Game(MicroBit *mb)
{
    uBit=mb;
    this->uBit->sleep(100);
    this->img = ManagedBuffer(LCD_WIDTH * LCD_HEIGHT * 2);
    this->pauseMenu = ManagedBuffer(LCD_WIDTH * LCD_HEIGHT * 2);
    this->lcd = new Adafruit_ST7735(*uBit, LCD_PIN_CS, LCD_PIN_DC, LCD_PIN_RST, LCD_PIN_MOSI, LCD_PIN_MISO, LCD_PIN_SCLK);
    this->lcd->initR(INITR_GREENTAB);
    for(int i=0;i<2;i++)
        this->npcs[i]=NPC();
    
    this->startGame();
}

void Game::setBackground(ManagedBuffer buf)
{
    uint16_t *p = (uint16_t *) &buf[0];
    while(p < (uint16_t *) &buf[buf.length()])
    {
        for(int i=0;i<15 && p < (uint16_t *) &buf[buf.length()];i++)
        {
            *p=LCD_GREEN_16_BIT;
            p++;
        }    
        for(int i=15;i<115 && p < (uint16_t *) &buf[buf.length()];i++)
        {
            *p=GREY;
            if(i==40 || i==65 || i==90)
                *p=ST77XX_WHITE;
            p++;
        }
        for(int i=115;i<128 && p < (uint16_t *) &buf[buf.length()];i++)
        {
            *p=LCD_GREEN_16_BIT;
            p++;
        }
    }
}

void Game::updateScreen(ManagedBuffer buf,Adafruit_ST7735 *lcd)
{
    lcd->sendData(0, 0, LCD_WIDTH, LCD_HEIGHT, buf.getBytes());
    uBit->sleep(500);
}

void Game::initGame(ManagedBuffer buf,Adafruit_ST7735 *lcd)
{
    setBackground(buf);
    updateScreen(buf,lcd);
}

void Game::drawCar(ManagedBuffer buf,int lane)
{
    uint16_t *p = (uint16_t *) &buf[0];
    p+=(128*128);
    uint16_t value=ST77XX_WHITE;
    int start;
    if(lane==0)
        start=17;
    else if(lane==1)
        start=42;
    else if(lane==2)
        start=67;
    else start=92;
    p+=start;
    p+=3;
    for(int k=0;k<3;k++)
    {
        for(int i=0;i<15;i++)
        {
            *p=value;
            p++;
        }
        p+=113;
    }
    p--;
    for(int k=0;k<3;k++)
    {
        for(int i=0;i<17;i++)
        {
            *p=value;
            p++;
        }
        p+=111;
    }
    
    p++;
    for(int k=0;k<10;k++)
    {
        for(int i=0;i<15;i++)
        {
            *p=value;
            p++;
        }
        p+=113;
    }
    p--;
    for(int k=0;k<3;k++)
    {
        for(int i=0;i<17;i++)
        {
            *p=value;
            p++;
        }
        p+=111;
    }
    p++;
    for(int k=0;k<5;k++)
    {
        for(int i=0;i<15;i++)
        {
            *p=value;
            p++;
        }
        p+=113;
    }
    p+=3;
    for(int i=0;i<7;i++)
    {
        *p=value;
        p++;
    }
}

void Game::eraseCar(ManagedBuffer buf,int lane,int offset)
{
    uint16_t *p = (uint16_t *) &buf[0];
    p+=(128*offset);
    uint16_t value=GREY;
    int start;
    if(lane==0)
        start=17;
    else if(lane==1)
        start=42;
    else if(lane==2)
        start=67;
    else start=92;
    p+=start;
    p+=3;
    for(int k=0;k<3;k++)
    {
        for(int i=0;i<15;i++)
        {
            *p=value;
            p++;
        }
        p+=113;
    }
    p--;
    for(int k=0;k<3;k++)
    {
        for(int i=0;i<17;i++)
        {
            *p=value;
            p++;
        }
        p+=111;
    }
    
    p++;
    for(int k=0;k<10;k++)
    {
        for(int i=0;i<15;i++)
        {
            *p=value;
            p++;
        }
        p+=113;
    }
    p--;
    for(int k=0;k<3;k++)
    {
        for(int i=0;i<17;i++)
        {
            *p=value;
            p++;
        }
        p+=111;
    }
    p++;
    for(int k=0;k<5;k++)
    {
        for(int i=0;i<15;i++)
        {
            *p=value;
            p++;
        }
        p+=113;
    }
    p+=3;
    for(int i=0;i<7;i++)
    {
        *p=value;
        p++;
    }
}

void Game::drawNPC(ManagedBuffer buf,int lane,int startRow)
{
    uint16_t *p = (uint16_t *) &buf[0];
    p+=(128*startRow);
    uint16_t value=LCD_BLUE_16_BIT;
    int start;
    if(lane==0)
        start=17;
    else if(lane==1)
        start=42;
    else if(lane==2)
        start=67;
    else start=92;
    p+=start;
    p+=3;
    for(int k=0;k<3;k++)
    {
        for(int i=0;i<15;i++)
        {
            *p=value;
            p++;
        }
        p+=113;
    }
    p--;
    for(int k=0;k<3;k++)
    {
        for(int i=0;i<17;i++)
        {
            *p=value;
            p++;
        }
        p+=111;
    }
    
    p++;
    for(int k=0;k<10;k++)
    {
        for(int i=0;i<15;i++)
        {
            *p=value;
            p++;
        }
        p+=113;
    }
    p--;
    for(int k=0;k<3;k++)
    {
        for(int i=0;i<17;i++)
        {
            *p=value;
            p++;
        }
        p+=111;
    }
    p++;
    for(int k=0;k<5;k++)
    {
        for(int i=0;i<15;i++)
        {
            *p=value;
            p++;
        }
        p+=113;
    }
    p+=3;
    for(int i=0;i<7;i++)
    {
        *p=value;
        p++;
    }
}

void Game::initNpcs(int size, NPC npcs[])
{
    srand(time(NULL));
    for(int i=0;i<size;i++)
    {
        npcs[i].setSpawned(false);
        npcs[i].setOffset(2);
        npcs[i].setLane(rand()%4);
        npcs[i].setDelay(rand()%10+2);
    }
}

bool Game::checkLanes(NPC npcs[], int lane, int size)
{
    for(int i=0;i<size;i++)
        if(npcs[i].getLane()==lane && npcs[i].getOffset()<50)
        {
            return false;
        }
    return true;
}

void Game::updateNPCS(ManagedBuffer img, NPC npcs[], int size,int iterations)
{
    srand(time(NULL));
    for(int i=0;i<size;i++)
    {
        if(npcs[i].getSpawned()==false && iterations>=npcs[i].getDelay())
        {
            npcs[i].setOffset(2);
            drawNPC(img,npcs[i].getLane(),npcs[i].getOffset());
            npcs[i].setSpawned(true);
        }
        else{
            checkCollision(npcs[i]);
            eraseCar(img,npcs[i].getLane(),npcs[i].getOffset());
            npcs[i].setOffset(npcs[i].getOffset()+5);
            if(npcs[i].getOffset()>132)
            {
                int newLane=rand()%4;
                while(!checkLanes(npcs,newLane,size))
                    newLane=rand()%4;
                npcs[i].setOffset(2);
                npcs[i].setDelay(rand()%10+iterations);
                npcs[i].setLane(newLane);
                npcs[i].setSpawned(false);
            }
            if(npcs[i].getSpawned()==true)
                drawNPC(img,npcs[i].getLane(),npcs[i].getOffset());
        }
        
    }
    
}

void Game::checkCollision(NPC npc)
{
    if(npc.getLane()==this->prevLane)
    {
        if(npc.getOffset()+25>=128)
            paused=!paused,endGame();
    }
}

void Game::drawBackground(ManagedBuffer buf)
{
    uint16_t *p = (uint16_t *) &buf[0];
    uint16_t value=LCD_GREEN_16_BIT;
    while(p<(uint16_t *) &buf[buf.length()])
    {
        *p=value;
        p++;
    }
}

void Game::drawPauseScreen(ManagedBuffer img)
{
    uint16_t *p = (uint16_t *) &img[0];
    uint16_t value=ST77XX_WHITE;
    drawBackground(img);
    p+=(128*70);
    p+=59;
    for(int k=0;k<20;k++)
    {
        for(int i=0;i<3;i++)
        {
            *p=value;
            p++;
        }
        p+=5;
        for(int i=0;i<3;i++)
        {
            *p=value;
            p++;
        }
        p+=117;
    }
}

void Game::drawStartScreen(ManagedBuffer buf)
{
    drawBackground(buf);
    uint16_t *p = (uint16_t *) &buf[0];
    uint16_t value=ST77XX_WHITE;
    p+=(128*70);
    p+=39;
    for(int i=0;i<20;i++)
    {
        *p=value;
        p++;
    }
    p+=10;
    for(int i=0;i<20;i++)
    {
        *p=value;
        p++;
    }
    p+=78;
    for(int k=0;k<13;k++)
    {
        *p=value;
        p+=30;
        *p=value;
        p+=20;
        *p=value;
        p+=78;
    }
    *p=value;
    p+=10;
    for(int i=0;i<10;i++)
    {
        *p=value;
        p++;
    }
    p+=10;
    *p=value;
    p+=20;
    *p=value;
    p+=78;
    for(int k=0;k<14;k++)
    {
        *p=value;
        p+=20;
        *p=value;
        p+=10;
        *p=value;
        p+=20;
        *p=value;
        p+=78;
    }
    for(int i=0;i<20;i++)
    {
        *p=value;
        p++;
    }
    p+=10;
    for(int i=0;i<20;i++)
    {
        *p=value;
        p++;
    }
}

void Game::startGame()
{
    drawStartScreen(this->pauseMenu);
    updateScreen(pauseMenu,lcd);
    while(uBit->io.P9.getDigitalValue());
    mainGameLoop();
}

void Game::endGame()
{
    uBit->serial.printf("Score: %d\n");
    startGame();
}

void Game::pauseGame()
{
    drawPauseScreen(pauseMenu);
    updateScreen(pauseMenu,lcd);
}

void Game::mainGameLoop()
{
    this->initGame(img,lcd);
    this->drawCar(img,1);
    this->updateScreen(img,lcd);
    this->count=0;
    this->prevLane=1;
    this->score=0;
    this->moved=0;
    this->initNpcs(2,npcs);
    this->paused=false;
    this->iterations=0;
    while(1)
    {
        if(!uBit->io.P9.getDigitalValue())
        {
            paused=!paused;
            if(paused==true)
                pauseGame();
            else updateScreen(img,lcd);
            uBit->sleep(400);
        }
        if(paused==false)
        {
            iterations=(iterations==100?0:iterations);
            moved=0;
            if(!uBit->io.P11.getDigitalValue())
                count=(count==0?0:count-1),moved=1;
            else if(!uBit->io.P16.getDigitalValue())
                count=(count==3?3:count+1),moved=1;
            updateNPCS(img,npcs,2,iterations);
            if(moved)
            {
                eraseCar(img,prevLane,128);
                prevLane=count;
                drawCar(img,count);
            }
            updateScreen(img,lcd);
            iterations++;
            score++;
        }

    }
}


Game::~Game()
{
    delete lcd;
}