#ifndef GAME_H

#define GAME_H

#include "MicroBit.h"
#include "Adafruit_ST7735.h"
#include <stdlib.h> 
#include <time.h> 
#include "NPC.h"

#define LCD_PIN_CS      2
#define LCD_PIN_DC      1
#define LCD_PIN_RST     0
#define LCD_PIN_MOSI    15
#define LCD_PIN_MISO    14
#define LCD_PIN_SCLK    13

// Define the dimensions of the display.
#define LCD_WIDTH       128
#define LCD_HEIGHT      160

// Constants for colors
#define LCD_RED_16_BIT      0xFC00
#define LCD_BLUE_16_BIT     0x03E0
#define LCD_GREEN_16_BIT    0x001F
#define LCD_WHITE           LCD_BLUE_16_BIT|LCD_GREEN_16_BIT|LCD_RED_16_BIT
#define ST77XX_BLACK 0x0000
#define ST77XX_WHITE 0xFFFF
#define ST77XX_RED 0xF800
#define ST77XX_GREEN 0x07E0
#define ST77XX_BLUE 0x001F
#define ST77XX_CYAN 0x07FF
#define ST77XX_MAGENTA 0xF81F
#define ST77XX_YELLOW 0xFFE0
#define ST77XX_ORANGE 0xFC00
#define GREY  (LCD_RED_16_BIT + LCD_GREEN_16_BIT + LCD_BLUE_16_BIT) / 3


/**
 * The class controlling all the game components.
*/
class Game{
private:
    MicroBit *uBit;
    ManagedBuffer img;
    ManagedBuffer pauseMenu;
    Adafruit_ST7735 *lcd;
    int count;
    int prevLane;
    int score;
    int moved;
    NPC npcs[2];
    bool paused;
    int iterations;
public:
    Game(MicroBit *mb);
    void setBackground(ManagedBuffer buf);
    void updateScreen(ManagedBuffer buf,Adafruit_ST7735 *lcd);
    void initGame(ManagedBuffer buf,Adafruit_ST7735 *lcd);
    void drawCar(ManagedBuffer buf,int lane);
    void eraseCar(ManagedBuffer buf,int lane,int offset);
    void drawNPC(ManagedBuffer buf,int lane,int startRow);
    void initNpcs(int size, NPC npcs[]);
    bool checkLanes(NPC npcs[], int lane, int size);
    void updateNPCS(ManagedBuffer img, NPC npcs[], int size,int iterations);
    void drawBackground(ManagedBuffer buf);
    void drawPauseScreen(ManagedBuffer img);
    void mainGameLoop();
    void pauseGame();
    void checkCollision(NPC npc);
    void startGame();
    void drawStartScreen(ManagedBuffer buf);
    void endGame();
    ~Game();
};

#endif